##############################################################################
# uniinput.tcl - unicode input tool
#
# Copyright (C) 2002 Reiji SUZUKI
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
##############################################################################
#


 #!/bin/sh
 # use -*-Tcl-*- \
 exec wish "$0"
source "./balloon.tcl"

proc init {} {
global ary
global maxl
        set fn "./uniinput_config.dat"
	set f [open $fn]
	fconfigure $f -encoding "utf-8"
	regsub -all \uFEFF [read $f [file size $fn]] "" text
	set i 0
	foreach line [split $text \n] {
	    set lines($i) "$line"
	    set j 0
	    foreach res [split $line ","] {
	    set ary($i,$j) "$res"
	        incr j
	    }
            incr i
	}
	set maxl $i
	close $f
	
}

proc textCopy w {
    if {![catch {set data [selection get -displayof $w]}]} {
	clipboard clear -displayof $w
	clipboard append -displayof $w $data
    }
}

proc textCopy_all w {
    if {![catch {set data [$w get 1.0 end]}]} {
	clipboard clear -displayof $w
	clipboard append -displayof $w $data
    }
}

 proc keyboard {w args} {
    global ary
    destroy $w
    frame $w
    array set opts {-keys {0xA0-0xFF} -keysperline 20}
    array set opts $args ;# no errors checked
    set klist {}; set n 0
    foreach i [clist2list $opts(-keys)] {
	set cn c
	if [regexp {([^-]+):([^-]+)} $i -> from to] {
		set c [format "%c%c" $from $to]
		set cmd "$opts(-receiver) insert insert [list $c]"
		button $w.k$i -font "\"$ary(0,0)\" $ary(0,1)"  -text $c -command $cmd  -padx 5 -pady 0
		set_balloon $w.k$i [format "%#x%#x" $from $to]
	} else {
		set c [format %c $i]
		set cmd "$opts(-receiver) insert insert [list $c]"
		button $w.k$i -font "\"$ary(0,0)\" $ary(0,1)"  -text $c -command $cmd  -padx 5 -pady 0
		set_balloon $w.k$i [format "%#x" $i]
	}
	set cmd "$opts(-receiver) insert insert [list $c]"
	if {$i>=0x5D0 && $i<=0x6FF} {
	    append cmd ";$opts(-receiver) mark set insert {insert - 1 chars}"
	}

	lappend klist $w.k$i
	if {[incr n]==$opts(-keysperline)} {
	    eval grid $klist -sticky news
	    set n 0; set klist {}
	}
    }
    if [llength $klist] {eval grid $klist -sticky news}
    pack $w -side bottom
    set w ;# return widget pathname, as the others do
 }

 proc clist2list {clist} {
    #-- clist: compact integer list w.ranges, e.g. {1-5 7 9-11}
    set res {}
    foreach i $clist {
	if [regexp {([^-]+)-([^-]+)} $i -> from to] {
	    for {set j [expr $from]} {$j<=[expr $to]} {incr j} {
		lappend res $j
	    }
	} else {if [regexp {([^-]+):([^-]+)} $i -> from to] {
		lappend res "$from:$to"
	} else {lappend res [expr $i]}
	}
    }
    set res
 }
 proc file:open {w} {
global ary
    set fn [tk_getOpenFile]
    if [string length $fn] {
	$w delete 1.0 end
	set f [open $fn]
	fconfigure $f -encoding $::Encoding
	regsub -all \uFEFF [read $f [file size $fn]] "" text
	foreach line [split $text \n] {
	    $w insert end $line\n
	}
	close $f
    }
 }
 proc file:save {w} {
    set fn [tk_getSaveFile]
    if [string length $fn] {
	set f [open $fn w]
	fconfigure $f -encoding $::Encoding
	if {$::Encoding=="unicode"} {
	    puts -nonewline $f \uFEFF
	}
	puts -nonewline $f [$w get 1.0 end-2c]
	close $f
    }
 }
 ##################################################################### #
 init
 menu .menu
 . config -menu .menu

 menu .menu.file -tearoff 0
 .menu add cascade -label File -menu .menu.file
 .menu.file add command -label Open... -command {file:open .t}
 .menu.file add command -label Save... -command {file:save .t}
 .menu.file add separator
 .menu.file add command -label Exit -command exit

 menu .menu.enc -tearoff 0
 .menu add cascade -label Encoding -menu .menu.enc
 foreach i {
    ascii cp1252 euc-jp iso2022-jp iso8859-1 iso8859-2 iso8859-3
    iso8859-4 iso8859-5 iso8859-6 iso8859-7 iso8859-8
    jis0208 koi8-r shiftjis utf-8 unicode
 } {
   .menu.enc add radio -label $i -variable ::Encoding -value $i
 }
 set ::Encoding [encoding system]

 menu .menu.lang -tearoff 0
 .menu add cascade -label Language -menu .menu.lang
 for {set i 4} {$i < $maxl} { incr i} {
    .menu.lang add command -label $ary($i,0) -command \
	    [list keyboard .kbd -keys $ary($i,1) -receiver .t]
 }

 keyboard .kbd -receiver .t
 pack [text .t -font "\"$ary(0,0)\" $ary(0,1)" -width 70 -height 10] -fill both -expand 1
 focus .t


menu .popup -tearoff no
.popup add command -label $ary(1,0) -underline 3 -command {textCopy .t}
.popup add command -label $ary(2,0) -underline 3 -command {textCopy_all .t}
.popup add command -label $ary(3,0) -underline 3 -command {.t delete 1.0 end}
bind . <3> { tk_popup .popup %X %Y }

